@@ -1,38 +1,45 @@
 /*
 htop - ScreenManager.c
 (C) 2004-2011 Hisham H. Muhammad
-Released under the GNU GPLv2, see the COPYING file
+Released under the GNU GPLv2+, see the COPYING file
 in the source distribution for its full text.
 */
 
+#include "config.h" // IWYU pragma: keep
+
 #include "ScreenManager.h"
 
 #include <assert.h>
 #include <stdbool.h>
 #include <stdlib.h>
+#include <string.h>
 #include <sys/time.h>
 
 #include "CRT.h"
 #include "FunctionBar.h"
+#include "Machine.h"
+#include "Macros.h"
 #include "Object.h"
-#include "ProcessList.h"
+#include "Platform.h"
+#include "Process.h"
 #include "ProvideCurses.h"
+#include "Settings.h"
+#include "Table.h"
 #include "XUtils.h"
 
 
-ScreenManager* ScreenManager_new(Header* header, const Settings* settings, const State* state, bool owner) {
+ScreenManager* ScreenManager_new(Header* header, Machine* host, State* state, bool owner) {
    ScreenManager* this;
    this = xMalloc(sizeof(ScreenManager));
    this->x1 = 0;
-   this->y1 = header->height;
+   this->y1 = 0;
    this->x2 = 0;
    this->y2 = -1;
    this->panels = Vector_new(Class(Panel), owner, DEFAULT_SIZE);
    this->panelCount = 0;
    this->header = header;
-   this->settings = settings;
+   this->host = host;
    this->state = state;
-   this->owner = owner;
    this->allowFocusChange = true;
    return this;
 }
@@ -42,61 +49,83 @@ void ScreenManager_delete(ScreenManager* this) {
    free(this);
 }
 
-inline int ScreenManager_size(ScreenManager* this) {
+inline int ScreenManager_size(const ScreenManager* this) {
    return this->panelCount;
 }
 
 void ScreenManager_add(ScreenManager* this, Panel* item, int size) {
+   ScreenManager_insert(this, item, size, Vector_size(this->panels));
+}
+
+static int header_height(const ScreenManager* this) {
+   if (this->state->hideMeters)
+      return 0;
+
+   if (this->header)
+      return this->header->height;
+
+   return 0;
+}
+
+void ScreenManager_insert(ScreenManager* this, Panel* item, int size, int idx) {
    int lastX = 0;
-   if (this->panelCount > 0) {
-      Panel* last = (Panel*) Vector_get(this->panels, this->panelCount - 1);
+   if (idx > 0) {
+      const Panel* last = (const Panel*) Vector_get(this->panels, idx - 1);
       lastX = last->x + last->w + 1;
    }
-   int height = LINES - this->y1 + this->y2;
-   if (size > 0) {
-      Panel_resize(item, size, height);
-   } else {
-      Panel_resize(item, COLS - this->x1 + this->x2 - lastX, height);
+   int height = LINES - this->y1 - header_height(this) + this->y2;
+   if (size <= 0) {
+      size = COLS - this->x1 + this->x2 - lastX;
    }
-   Panel_move(item, lastX, this->y1);
-   Vector_add(this->panels, item);
+   Panel_resize(item, size, height);
+   Panel_move(item, lastX, this->y1 + header_height(this));
+   if (idx < this->panelCount) {
+      for (int i =  idx + 1; i <= this->panelCount; i++) {
+         Panel* p = (Panel*) Vector_get(this->panels, i);
+         Panel_move(p, p->x + size, p->y);
+      }
+   }
+   Vector_insert(this->panels, idx, item);
    item->needsRedraw = true;
    this->panelCount++;
 }
 
 Panel* ScreenManager_remove(ScreenManager* this, int idx) {
    assert(this->panelCount > idx);
+   int w = ((Panel*) Vector_get(this->panels, idx))->w;
    Panel* panel = (Panel*) Vector_remove(this->panels, idx);
    this->panelCount--;
+   if (idx < this->panelCount) {
+      for (int i = idx; i < this->panelCount; i++) {
+         Panel* p = (Panel*) Vector_get(this->panels, i);
+         Panel_move(p, p->x - w, p->y);
+      }
+   }
    return panel;
 }
 
-void ScreenManager_resize(ScreenManager* this, int x1, int y1, int x2, int y2) {
-   this->x1 = x1;
-   this->y1 = y1;
-   this->x2 = x2;
-   this->y2 = y2;
+void ScreenManager_resize(ScreenManager* this) {
+   int y1_header = this->y1 + header_height(this);
    int panels = this->panelCount;
    int lastX = 0;
    for (int i = 0; i < panels - 1; i++) {
       Panel* panel = (Panel*) Vector_get(this->panels, i);
-      Panel_resize(panel, panel->w, LINES - y1 + y2);
-      Panel_move(panel, lastX, y1);
+      Panel_resize(panel, panel->w, LINES - y1_header + this->y2);
+      Panel_move(panel, lastX, y1_header);
       lastX = panel->x + panel->w + 1;
    }
    Panel* panel = (Panel*) Vector_get(this->panels, panels - 1);
-   Panel_resize(panel, COLS - x1 + x2 - lastX, LINES - y1 + y2);
-   Panel_move(panel, lastX, y1);
+   Panel_resize(panel, COLS - this->x1 + this->x2 - lastX, LINES - y1_header + this->y2);
+   Panel_move(panel, lastX, y1_header);
 }
 
-static void checkRecalculation(ScreenManager* this, double* oldTime, int* sortTimeout, bool* redraw, bool* rescan, bool* timedOut) {
-   ProcessList* pl = this->header->pl;
+static void checkRecalculation(ScreenManager* this, double* oldTime, int* sortTimeout, bool* redraw, bool* rescan, bool* timedOut, bool* force_redraw) {
+   Machine* host = this->host;
 
-   struct timeval tv;
-   gettimeofday(&tv, NULL);
-   double newTime = ((double)tv.tv_sec * 10) + ((double)tv.tv_usec / 100000);
+   Platform_gettime_realtime(&host->realtime, &host->realtimeMs);
+   double newTime = ((double)host->realtime.tv_sec * 10) + ((double)host->realtime.tv_usec / 100000);
 
-   *timedOut = (newTime - *oldTime > this->settings->delay);
+   *timedOut = (newTime - *oldTime > host->settings->delay);
    *rescan |= *timedOut;
 
    if (newTime < *oldTime) {
@@ -105,34 +134,99 @@ static void checkRecalculation(ScreenManager* this, double* oldTime, int* sortTi
 
    if (*rescan) {
       *oldTime = newTime;
-      ProcessList_scan(pl, this->state->pauseProcessUpdate);
-      if (!this->state->pauseProcessUpdate && (*sortTimeout == 0 || this->settings->treeView)) {
-         ProcessList_sort(pl);
+      int oldUidDigits = Process_uidDigits;
+      if (!this->state->pauseUpdate && (*sortTimeout == 0 || host->settings->ss->treeView)) {
+         host->activeTable->needsSort = true;
          *sortTimeout = 1;
       }
+      // sample current values for system metrics and processes if not paused
+      Machine_scan(host);
+      if (!this->state->pauseUpdate)
+         Machine_scanTables(host);
+
+      // always update header, especially to avoid gaps in graph meters
+      Header_updateData(this->header);
+      // force redraw if the number of UID digits was changed
+      if (Process_uidDigits != oldUidDigits) {
+         *force_redraw = true;
+      }
       *redraw = true;
    }
    if (*redraw) {
-      ProcessList_rebuildPanel(pl);
-      Header_draw(this->header);
+      Table_rebuildPanel(host->activeTable);
+      if (!this->state->hideMeters)
+         Header_draw(this->header);
    }
    *rescan = false;
 }
 
+static inline bool drawTab(const int* y, int* x, int l, const char* name, bool cur) {
+   attrset(CRT_colors[cur ? SCREENS_CUR_BORDER : SCREENS_OTH_BORDER]);
+   mvaddch(*y, *x, '[');
+   (*x)++;
+   if (*x >= l)
+      return false;
+   int nameLen = strlen(name);
+   int n = MINIMUM(l - *x, nameLen);
+   attrset(CRT_colors[cur ? SCREENS_CUR_TEXT : SCREENS_OTH_TEXT]);
+   mvaddnstr(*y, *x, name, n);
+   *x += n;
+   if (*x >= l)
+      return false;
+   attrset(CRT_colors[cur ? SCREENS_CUR_BORDER : SCREENS_OTH_BORDER]);
+   mvaddch(*y, *x, ']');
+   *x += 1 + SCREEN_TAB_COLUMN_GAP;
+   if (*x >= l)
+      return false;
+   return true;
+}
+
+static void ScreenManager_drawScreenTabs(ScreenManager* this) {
+   Settings* settings = this->host->settings;
+   ScreenSettings** screens = settings->screens;
+   int cur = settings->ssIndex;
+   int l = COLS;
+   Panel* panel = (Panel*) Vector_get(this->panels, 0);
+   int y = panel->y - 1;
+   int x = SCREEN_TAB_MARGIN_LEFT;
+
+   if (this->name) {
+      drawTab(&y, &x, l, this->name, true);
+      return;
+   }
+
+   for (int s = 0; screens[s]; s++) {
+      bool ok = drawTab(&y, &x, l, screens[s]->heading, s == cur);
+      if (!ok) {
+         break;
+      }
+   }
+   attrset(CRT_colors[RESET_COLOR]);
+}
+
 static void ScreenManager_drawPanels(ScreenManager* this, int focus, bool force_redraw) {
+   Settings* settings = this->host->settings;
+   if (settings->screenTabs) {
+      ScreenManager_drawScreenTabs(this);
+   }
    const int nPanels = this->panelCount;
    for (int i = 0; i < nPanels; i++) {
       Panel* panel = (Panel*) Vector_get(this->panels, i);
-      Panel_draw(panel, force_redraw, i == focus, !((panel == this->state->panel) && this->state->hideProcessSelection), State_hideFunctionBar(this->state));
+      Panel_draw(panel,
+                 force_redraw,
+                 i == focus,
+                 panel != (Panel*)this->state->mainPanel || !this->state->hideSelection,
+                 State_hideFunctionBar(this->state));
       mvvline(panel->y, panel->x + panel->w, ' ', panel->h + (State_hideFunctionBar(this->state) ? 1 : 0));
    }
 }
 
-void ScreenManager_run(ScreenManager* this, Panel** lastFocus, int* lastKey) {
+void ScreenManager_run(ScreenManager* this, Panel** lastFocus, int* lastKey, const char* name) {
    bool quit = false;
    int focus = 0;
 
    Panel* panelFocus = (Panel*) Vector_get(this->panels, focus);
+   Settings* settings = this->host->settings;
 
    double oldTime = 0.0;
 
@@ -146,22 +240,30 @@ void ScreenManager_run(ScreenManager* this, Panel** lastFocus, int* lastKey) {
    int sortTimeout = 0;
    int resetSortTimeout = 5;
 
+   this->name = name;
+
    while (!quit) {
       if (this->header) {
-         checkRecalculation(this, &oldTime, &sortTimeout, &redraw, &rescan, &timedOut);
+         checkRecalculation(this, &oldTime, &sortTimeout, &redraw, &rescan, &timedOut, &force_redraw);
       }
 
       if (redraw || force_redraw) {
          ScreenManager_drawPanels(this, focus, force_redraw);
          force_redraw = false;
+         if (this->host->iterationsRemaining != -1) {
+            if (!--this->host->iterationsRemaining) {
+               quit = true;
+               continue;
+            }
+         }
       }
 
       int prevCh = ch;
-      set_escdelay(25);
-      ch = getch();
+      ch = Panel_getCh(panelFocus);
 
       HandlerResult result = IGNORED;
-      if (ch == KEY_MOUSE && this->settings->enableMouse) {
+#ifdef HAVE_GETMOUSE
+      if (ch == KEY_MOUSE && settings->enableMouse) {
          ch = ERR;
          MEVENT mevent;
          int ok = getmouse(&mevent);
@@ -176,12 +278,15 @@ void ScreenManager_run(ScreenManager* this, Panel** lastFocus, int* lastKey) {
                         if (mevent.y == panel->y) {
                            ch = EVENT_HEADER_CLICK(mevent.x - panel->x);
                            break;
+                        } else if (settings->screenTabs && mevent.y == panel->y - 1) {
+                           ch = EVENT_SCREEN_TAB_CLICK(mevent.x);
+                           break;
                         } else if (mevent.y > panel->y && mevent.y <= panel->y + panel->h) {
                            ch = KEY_MOUSE;
                            if (panel == panelFocus || this->allowFocusChange) {
                               focus = i;
                               panelFocus = panel;
-                              Object* oldSelection = Panel_getSelected(panel);
+                              const Object* oldSelection = Panel_getSelected(panel);
                               Panel_setSelected(panel, mevent.y - panel->y + panel->scrollV - 1);
                               if (Panel_getSelected(panel) == oldSelection) {
                                  ch = KEY_RECLICK;
@@ -201,8 +306,10 @@ void ScreenManager_run(ScreenManager* this, Panel** lastFocus, int* lastKey) {
             }
          }
       }
+#endif
       if (ch == ERR) {
-         sortTimeout--;
+         if (sortTimeout > 0)
+            sortTimeout--;
          if (prevCh == ch && !timedOut) {
             closeTimeout++;
             if (closeTimeout == 100) {
@@ -214,12 +321,14 @@ void ScreenManager_run(ScreenManager* this, Panel** lastFocus, int* lastKey) {
          redraw = false;
          continue;
       }
+
       switch (ch) {
          case KEY_ALT('H'): ch = KEY_LEFT; break;
          case KEY_ALT('J'): ch = KEY_DOWN; break;
          case KEY_ALT('K'): ch = KEY_UP; break;
          case KEY_ALT('L'): ch = KEY_RIGHT; break;
       }
+
       redraw = true;
       if (Panel_eventHandlerFn(panelFocus)) {
          result = Panel_eventHandler(panelFocus, ch);
@@ -233,6 +342,10 @@ void ScreenManager_run(ScreenManager* this, Panel** lastFocus, int* lastKey) {
       if (result & REDRAW) {
          force_redraw = true;
       }
+      if (result & RESIZE) {
+         ScreenManager_resize(this);
+         force_redraw = true;
+      }
       if (result & RESCAN) {
          rescan = true;
          sortTimeout = 0;
@@ -247,9 +360,12 @@ void ScreenManager_run(ScreenManager* this, Panel** lastFocus, int* lastKey) {
       switch (ch) {
       case KEY_RESIZE:
       {
-         ScreenManager_resize(this, this->x1, this->y1, this->x2, this->y2);
+         ScreenManager_resize(this);
          continue;
       }
+      case KEY_FOCUS_IN:
+      case KEY_FOCUS_OUT:
+         break;
       case KEY_LEFT:
       case KEY_CTRL('B'):
          if (this->panelCount < 2) {
@@ -291,6 +407,11 @@ tryRight:
             goto tryRight;
          }
 
+         break;
+      case '#':
+         this->state->hideMeters = !this->state->hideMeters;
+         ScreenManager_resize(this);
+         force_redraw = true;
          break;
       case 27:
       case 'q':
