@@ -16,6 +16,7 @@ in the source distribution for its full text.
 #include "ClockMeter.h"
 #include "DateMeter.h"
 #include "DateTimeMeter.h"
+#include "FileDescriptorMeter.h"
 #include "HostnameMeter.h"
 #include "LoadAverageMeter.h"
 #include "Macros.h"
@@ -70,6 +71,7 @@ const MeterClass* const Platform_meterTypes[] = {
    &RightCPUs4Meter_class,
    &LeftCPUs8Meter_class,
    &RightCPUs8Meter_class,
+   &FileDescriptorMeter_class,
    &BlankMeter_class,
    NULL
 };
@@ -100,8 +102,8 @@ void Platform_getLoadAverage(double* one, double* five, double* fifteen) {
    *fifteen = 0;
 }
 
-int Platform_getMaxPid(void) {
-   return 1;
+pid_t Platform_getMaxPid(void) {
+   return INT_MAX;
 }
 
 double Platform_setCPUValues(Meter* this, unsigned int cpu) {
@@ -134,6 +136,11 @@ FileLocks_ProcessData* Platform_getProcessLocks(pid_t pid) {
    return NULL;
 }
 
+void Platform_getFileDescriptors(double* used, double* max) {
+   *used = 1337;
+   *max = 4711;
+}
+
 bool Platform_getDiskIO(DiskIOData* data) {
    (void)data;
    return false;
